// Copyright 2022 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package configuration

import (
	"bytes"
	"io"
	"testing"
)

type mockTestPostprocessingEventHandler struct {
	ChangeHook
}

func (m *mockTestPostprocessingEventHandler) Handle(event PostprocessingEvent) {
	// do nothing

}

func TestAddRemovePostprocessingHook(t *testing.T) {

	config := struct {
		Host string
	}{
		Host: "localhost",
	}

	s := New(config)

	var h PostprocessingHook
	h = &mockTestPostprocessingEventHandler{}
	s.OnPostprocessing(h)

	if len(s.hooks.postprocessing) != 1 {
		t.Error("Expected 1 got ", len(s.hooks.postprocessing))
	}

	s.RemoveOnPostprocessingHook(h)

	if len(s.hooks.postprocessing) != 0 {
		t.Error("Expected 0 got ", len(s.hooks.postprocessing))
	}

}

func TestPostprocessingOnChange(t *testing.T) {

	defaults := ConfigStruct2{
		A: "",
		B: false,
		C: nil,
		D: nil,
		F: 0,
		G: "",
	}

	s := New(defaults)
	s.SetMnemonic("test")

	var buf bytes.Buffer
	buf.WriteString("A: a\n")
	r := (io.Reader)(&buf)

	s.AddReader(r, Yaml)

	s.Import()

	if s.HasErrors() {
		t.Error("Expected not error", s.Errors())
	}

	c := s.Config()
	if c.A != "a" {
		t.Error("Expected \"a\" got ", c)
	}

	counter := 0
	var p PostprocessingHook
	p = &PostprocessingHandler{
		Callback: func(event PostprocessingEvent) {
			counter++
		},
	}

	s.OnPostprocessing(p)

	c.A = "x"
	s.SetConfig(c)

	if counter != 1 {
		t.Error("Expected 1 got ", counter)
	}

}
